﻿//using System.Data;
//using System.Text;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
    
namespace RenderMesh
{
    public partial class Form1 : Form
    {
        [STAThread]
        static void Main()
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
    
            using (Form1 frm = new Form1())
            {
                // Показать форму перед созданием движка
                frm.Show();
                // Создать и инициализировать графический движок
                // нашей функцией
                frm.InitializeGraphics();
                // Войти в цикл сообщений
                Application.Run(frm);
            }
        }
    
        public Form1()
        {
            InitializeComponent();
    
            // Устраняем мерцание окна при бесконечной перерисовке
            this.SetStyle(ControlStyles.AllPaintingInWmPaint |
                ControlStyles.Opaque, true);
            this.Click += new EventHandler(Form1_Click);
        }
    
        bool flagRotate = true;
        void Form1_Click(object sender, EventArgs e)
        {
            flagRotate = !flagRotate;
        }
    
        private Device device = null;
        private Mesh mesh = null;
    
        public void InitializeGraphics()
        {
            // Создание объекта и настройка параметров представления
            // Создать объект параметров представления
            PresentParameters presentParams = new PresentParameters();
            // Установить оконный режим
            presentParams.Windowed = true;
            // Сбрасывать содержимое буфера, если он не готов к представлению
            presentParams.SwapEffect = SwapEffect.Discard;
    
            // Создать объект устройства и сохранить ссылку на него
            device = new Device(0, DeviceType.Hardware, this,
                CreateFlags.SoftwareVertexProcessing, presentParams);
    
            // Генерация и подключение к устройству куба 
            // с помощью статического метода класса Mesh
            mesh = Mesh.Box(device, 2.0F, 2.0F, 2.0F);
        }
            
        protected override void OnPaint(PaintEventArgs e)
        {
            // Очистить цветом клиентскую область формы
            device.Clear(ClearFlags.Target,
                System.Drawing.Color.CornflowerBlue,
                1.0F, 0);
    
            // Вызов нашей функции установки камеры
            SetupCamera();
    
            // Сформировать сцену
            device.BeginScene();
    
            DrawCubes();
            device.EndScene();
    
            // Показать буфер кадра 
            device.Present();
    
            // Принудительно перерисовать
            this.Invalidate();
        }

        private float angle = 0;// Закрытый член класса
    
        // Установка камеры в сцену
        private void SetupCamera()
        {
            //Создание перспективы 
            device.Transform.Projection = Matrix.PerspectiveFovLH(
                (float)Math.PI / 4, // Угол зрения равен 45 градусов
                                    // Форматное соотношение сторон
                (float)this.ClientSize.Width / (float)this.ClientSize.Height, 
                1.0F,               // Ближний план
                100.0F);            // Дальний план
    
            //Добавление камеры 
            device.Transform.View = Matrix.LookAtLH(
                new Vector3(0, 0, 15.0F),    // Положение камеры
                new Vector3(),              // Положение объекта текущее
                new Vector3(0, 1, 0));      // Направление камеры
    
            // Освещение
            device.RenderState.Lighting = true;
    
            // Установить красный свет
            device.RenderState.Ambient = Color.Red;
    
            // Настроить общий свет
            device.Lights[0].Type = LightType.Directional;
            device.Lights[0].Diffuse = Color.White;
            device.Lights[0].Direction = new Vector3(0, -1, -1);
            device.Lights[0].Commit();
            device.Lights[0].Enabled = true;
        }
    
        private void DrawCubes()
        {
            // Создаем материал поверхности отражения
            // и добавляем его в устройство 
            Material boxMaterial = new Material();
            boxMaterial.Ambient = Color.White;
            boxMaterial.Diffuse = Color.White;
            device.Material = boxMaterial;
    
            //
            // Средний ряд
            //
            device.Transform.World = Matrix.RotationYawPitchRoll(
                angle / (float)Math.PI,
                angle / (float)Math.PI * 2.0f,
                angle / (float)Math.PI / 4.0f) *
                Matrix.Translation(0.0f, 0.0f, 0.0f);   // По центру
            mesh.DrawSubset(0);
    
            device.Transform.World = Matrix.RotationYawPitchRoll(
                angle / (float)Math.PI,
                angle / (float)Math.PI / 2.0f,
                angle / (float)Math.PI * 4.0f) *
                Matrix.Translation(4.0f, 0.0f, 0.0f);   // Сдвиг вправо
            mesh.DrawSubset(0);
    
            device.Transform.World = Matrix.RotationYawPitchRoll(
                angle / (float)Math.PI,
                angle / (float)Math.PI * 4.0f,
                angle / (float)Math.PI / 2.0f) *
                Matrix.Translation(-4.0f, 0.0f, 0.0f);   // Сдвиг влево
            mesh.DrawSubset(0);
    
            //
            // Нижний ряд
            //
            device.Transform.World = Matrix.RotationYawPitchRoll(
                angle / (float)Math.PI,
                angle / (float)Math.PI * 2.0f,
                angle / (float)Math.PI / 4.0f) *
                Matrix.Translation(0.0f, -4.0f, 0.0f);   // По центру
            mesh.DrawSubset(0);
    
            device.Transform.World = Matrix.RotationYawPitchRoll(
                angle / (float)Math.PI,
                angle / (float)Math.PI / 2.0f,
                angle / (float)Math.PI * 4.0f) *
                Matrix.Translation(4.0f, -4.0f, 0.0f);   // Сдвиг вправо
            mesh.DrawSubset(0);
    
            device.Transform.World = Matrix.RotationYawPitchRoll(
                angle / (float)Math.PI,
                angle / (float)Math.PI * 4.0f,
                angle / (float)Math.PI / 2.0f) *
                Matrix.Translation(-4.0f, -4.0f, 0.0f);   // Сдвиг влево
            mesh.DrawSubset(0);
    
            //
            // Верхний ряд
            //
            device.Transform.World = Matrix.RotationYawPitchRoll(
                angle / (float)Math.PI,
                angle / (float)Math.PI * 2.0f,
                angle / (float)Math.PI / 4.0f) *
                Matrix.Translation(0.0f, 4.0f, 0.0f);   // По центру
            mesh.DrawSubset(0);
    
            device.Transform.World = Matrix.RotationYawPitchRoll(
                angle / (float)Math.PI,
                angle / (float)Math.PI / 2.0f,
                angle / (float)Math.PI * 4.0f) *
                Matrix.Translation(4.0f, 4.0f, 0.0f);   // Сдвиг вправо
            mesh.DrawSubset(0);
    
            device.Transform.World = Matrix.RotationYawPitchRoll(
                angle / (float)Math.PI,
                angle / (float)Math.PI * 4.0f,
                angle / (float)Math.PI / 2.0f) *
                Matrix.Translation(-4.0f, 4.0f, 0.0f);   // Сдвиг влево
            mesh.DrawSubset(0);
    
            if (flagRotate)
                angle += 0.02F;
        }
    }
}