﻿//using System.Data;
//using System.Text;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
    
namespace RenderMesh
{
    public partial class Form2 : Form
    {
        [STAThread]
        static void Main()
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
    
            using (Form2 frm = new Form2())
            {
                // Показать форму перед созданием движка
                frm.Show();
                // Создать и инициализировать графический движок
                // нашей функцией
                frm.InitializeGraphics();
                // Войти в цикл сообщений
                Application.Run(frm);
            }
        }
    
        public Form2()
        {
            InitializeComponent();
    
            // Устраняем мерцание окна при бесконечной перерисовке
            this.SetStyle(ControlStyles.AllPaintingInWmPaint |
                ControlStyles.Opaque, true);
            this.Click += new EventHandler(Form2_Click);
        }
    
        bool flagRotate = true;
        void Form2_Click(object sender, EventArgs e)
        {
            flagRotate = !flagRotate;
        }
    
        private Device device = null;
        private Mesh mesh = null;
    
        // Ссылки на массивы для загрузки из X-файла
        private Material[] meshMaterials;
        private Texture[] meshTextures;
    
        public void InitializeGraphics()
        {
            // Создание объекта и настройка параметров представления
            // Создать объект параметров представления
            PresentParameters presentParams = new PresentParameters();
            // Установить оконный режим
            presentParams.Windowed = true;
            // Сбрасывать содержимое буфера, если он не готов к представлению
            presentParams.SwapEffect = SwapEffect.Discard;
    
            // Настройка буфера глубины для параметров устройства
            presentParams.EnableAutoDepthStencil = true;
            presentParams.AutoDepthStencilFormat =
                Microsoft.DirectX.Direct3D.DepthFormat.D16;
    
            // Создать объект устройства и сохранить ссылку на него
            device = new Device(0, DeviceType.Hardware, this,
                CreateFlags.SoftwareVertexProcessing, presentParams);
    
            // Генерация и подключение к устройству куба 
            // с помощью статического метода класса Mesh
            mesh = Mesh.Box(device, 2.0F, 2.0F, 2.0F);
    
            // Загрузить Mesh-объект из файла tiny.x
            mesh = LoadMesh("tiny.x");
        }
            
        protected override void OnPaint(PaintEventArgs e)
        {
            // Очистить цветом клиентскую область формы
            device.Clear(ClearFlags.Target 
                | ClearFlags.ZBuffer,
                System.Drawing.Color.CornflowerBlue,
                1.0F, 0);
    
            // Вызов нашей функции установки камеры
            SetupCamera();
    
            // Сформировать сцену
            device.BeginScene();
            DrawMesh();
            device.EndScene();
    
            // Показать буфер кадра 
            device.Present();
    
            // Принудительно перерисовать
            this.Invalidate();
        }
    
        private float angle = 0;// Закрытый член класса
    
        // Установка камеры в сцену
        private void SetupCamera()
        {
            //Создание перспективы 
            device.Transform.Projection = Matrix.PerspectiveFovLH(
                (float)Math.PI / 4, // Угол зрения равен 45 градусов
                                    // Форматное соотношение сторон
                (float)this.ClientSize.Width / (float)this.ClientSize.Height, 
                1.0F,               // Ближний план
                1000.0F);           // Дальний план
    
            //Добавление камеры 
            device.Transform.View = Matrix.LookAtLH(
                new Vector3(0, 0, 700.0F),  // Положение камеры
                new Vector3(),              // Положение объекта текущее
                new Vector3(0, 1, 0));      // Направление камеры
    
            // Освещение
            device.RenderState.Lighting = true;
    
            // Установить красный свет
            device.RenderState.Ambient = Color.Red;
    
            // Настроить общий свет
            device.Lights[0].Type = LightType.Directional;
            device.Lights[0].Diffuse = Color.White;
            device.Lights[0].Direction = new Vector3(0, -1, -1);
            device.Lights[0].Commit();
            device.Lights[0].Enabled = true;
        }
    
        private void DrawMesh()
        {
            // Вращаем объект
            device.Transform.World = Matrix.RotationYawPitchRoll(
                angle / (float)Math.PI,
                angle / (float)Math.PI * 2.0f,
                angle / (float)Math.PI / 4.0f) *
                Matrix.Translation(0.0f, 0.0f, 0.0f);
    
            // Загружаем в устройство материал и текстуру поверхности отражения
            for (int i = 0; i < meshMaterials.Length; i++)
            {
                device.Material = meshMaterials[i];
                device.SetTexture(0, meshTextures[i]);
                mesh.DrawSubset(0);
            }
    
            if (flagRotate)
                angle += 0.02F;
        }
    
        // Функция загрузки Mesh-объекта из файла
        private Mesh LoadMesh(string file)
        {
            ExtendedMaterial[] data; // Создаем ссылку на загружаемые данные
            // Загружаем объект из файла
            Mesh localMesh = Mesh.FromFile(file, MeshFlags.Managed, device, out data);
    
            // Если есть данные - разделяем и сохраняем их
            if ((data != null) && (data.Length > 0))
            {
                // Создаем массив материалов
                meshMaterials = new Material[data.Length];
                // Создаем массив текстур
                meshTextures = new Texture[data.Length];
    
                // Заполняем массив материалами и текстурами
                for (int i = 0; i < data.Length; i++)
                {
                    meshMaterials[i] = data[i].Material3D;
                    if ((data[i].TextureFilename != null)
                        && (data[i].TextureFilename != string.Empty))
                    {
                        // Структура существует, попробуем ее загрузить
                        meshTextures[i] = TextureLoader.FromFile(
                            device, data[i].TextureFilename);
                    }
                }
            }
    
            return localMesh;
        }
    }
}